#include "stdint.h"
#include "stralloc.h"
#include "sa_vector.h"
#include "constmap.h"
#include "alloc.h"
#include "byte.h"
#include "siphash.h"

static byte_t constmap_hashinitseed[32] = "constmap_hashinitseedrandombytes";

const char *constmap(const struct constmap *cm, const char *s, unsigned int len)
{
  constmap_hash h;
  int pos;
  const char *dataval;

  h = siphash24(s, len, constmap_hashinitseed);
  pos = cm->first[h & cm->mask];
  while (pos != -1) {
    if (h == cm->hash[pos]) {
      if (len == cm->inputlen[pos]) {
        if (!byte_diff(cm->input[pos], len, s)) {
          dataval = cm->input[pos];
          if (cm->flag_assoc_pair) {
            dataval += (len + 1);
          }
          return dataval;
        }
      }
    }
    pos = cm->next[pos];
  }
  return 0;
}

/* each entry of 'list' must be null terminated, cf sa_vector_0() */
unsigned int constmap_init(struct constmap *cm, const sa_vector *list, unsigned int flag_colon)
{
  const char *s;
  constmap_hash h;
  unsigned int pos;
  unsigned int num;
  unsigned int len;
  unsigned int keylen;

  cm->hash = 0;
  cm->first = cm->next = 0;
  cm->input = 0;
  cm->inputlen = 0;
  cm->num = num = list->len;
  cm->flag_assoc_pair = flag_colon;

  h = 64;
  while (h && (h < num)) {
    h += h;
  }
  cm->mask = h - 1;

  cm->first = (int*)alloc((unsigned int)sizeof(unsigned int) * (unsigned int)h);
  if (cm->first) {
    for (h = 0; h <= cm->mask; ++h) {
      cm->first[h] = -1;
    }
    cm->input = (const char**)alloc((unsigned int)sizeof(char*) * num);
    if (cm->input) {
      cm->inputlen = (unsigned int*)alloc((unsigned int)sizeof(unsigned int) * num);
      if (cm->inputlen) {
        cm->hash = (constmap_hash *)alloc((unsigned int)sizeof(constmap_hash) * num);
        if (cm->hash) {
          cm->next = (int *)alloc((unsigned int)sizeof(int) * num);
          if (cm->next) {
            for (pos = 0; pos < num; ++pos) {
              s = list->va[pos].s;
              keylen = len = list->va[pos].len - 1;  /* allow for null term */
              if (flag_colon) {
                for (keylen = 0; keylen < len; ++keylen) {
                  if (s[keylen] == ':') break;
                }
                if (keylen == len) continue;
              }
              cm->input[pos] = s;
              cm->inputlen[pos] = keylen;
              h = siphash24(s, keylen, constmap_hashinitseed);
              cm->hash[pos] = h;
              h &= cm->mask;
              cm->next[pos] = cm->first[h];
              cm->first[h] = (int)pos;
            }
            return 1;
          }
          alloc_free(cm->hash);
          cm->hash = 0;
        }
        alloc_free(cm->inputlen);
        cm->inputlen = 0;
      }
      alloc_free(cm->input);
      cm->input = 0;
    }
    alloc_free(cm->first);
    cm->first = 0;
  }
  return 0;
}

void constmap_free(struct constmap *cm)
{
  alloc_free(cm->next);
  cm->next = 0;

  alloc_free(cm->hash);
  cm->hash = 0;

  alloc_free(cm->inputlen);
  cm->inputlen = 0;

  alloc_free(cm->input);
  cm->input = 0;

  alloc_free(cm->first);
  cm->first = 0;
}
