#ifndef NEMO_IP4_H
#define NEMO_IP4_H

#ifdef __cplusplus
extern "C" {
#endif

typedef struct {
  byte_t d[4];
} ip4_address;

#define IP4_ADDRESS {{'\0','\0','\0','\0'}}

/*
 ip4 address syntax: (d = digit), no leading '0' required
   1. ddd.ddd.ddd.ddd
*/

unsigned int	ip4_fmt(const ip4_address *ip, void *out);
unsigned int	ip4_fmtiaa(const ip4_address *ip, void *out);
unsigned int	ip4_scan(ip4_address *ip, const char *s);
unsigned int	ip4_scanbracket(ip4_address *ip, const char *s);
unsigned int	ip4_port_scan(ip4_address *ip, unsigned int *port, const char *s);
unsigned int	ip4_mask_scan(ip4_address *ip, unsigned int *mask_depth, const char *s);

int		ip4_diff(const ip4_address *ip1, const ip4_address *ip2);
unsigned int	ip4_equal(const ip4_address *ip1, const ip4_address *ip2);
unsigned int	ip4_equal_mask(const ip4_address *ip1, unsigned int mask_depth, const ip4_address *ip2);

void		ip4_zero(ip4_address *ip);
void		ip4_copy(ip4_address *out, const ip4_address *in);

void		ip4_pack(const ip4_address *ip, void *d);
void		ip4_pack_uint32(const ip4_address *ip, uint32_t *u);
void		ip4_unpack(ip4_address *ip, const void *d);
void		ip4_unpack_uint32(ip4_address *ip, uint32_t u);

void		ip4_reverse(ip4_address *ip);

void		ip4_mask_0(ip4_address *ip, unsigned int mask_depth);

#define IP4_FMT 20
#define IP4_FMT_IAA 40

typedef struct {
  ip4_address *va;
  unsigned int len;
  unsigned int a;
} ip4_vector;

#define IP4_VECTOR {0,0,0}

static inline void ip4_vector_init(register ip4_vector *v)
{
  v->va = 0;
  v->a = v->len = 0;
}

/*  return: 1 = success, 0 = fail  */
unsigned int ip4_vector_realloc(ip4_vector *v, unsigned int n);
unsigned int ip4_vector_alloc(ip4_vector *v, unsigned int n);

static inline unsigned int ip4_vector_ready(register ip4_vector *v, register unsigned int n)
{
  if (v->va) {
    if (n <= v->a) return 1;
    return ip4_vector_realloc(v, n);
  }
  return ip4_vector_alloc(v, n);
}

static inline unsigned int ip4_vector_readyplus(register ip4_vector *v, register unsigned int n)
{
  if (v->va) {
    n += v->len;
    if (n <= v->a) return 1;
    return ip4_vector_realloc(v, n);
  }
  return ip4_vector_alloc(v, n);
}

unsigned int	ip4_vector_cat(ip4_vector *to, const ip4_vector *from);
unsigned int	ip4_vector_copy(ip4_vector *to, const ip4_vector *from);

unsigned int	ip4_vector_append(ip4_vector *v, const ip4_address *i);
unsigned int	ip4_vector_erase(ip4_vector *v);

void		ip4_vector_remove(ip4_vector *v, unsigned int pos, unsigned int len);
void		ip4_vector_swap(ip4_vector *v, unsigned int i, unsigned int j);
void		ip4_vector_sort(ip4_vector *v);
unsigned int	ip4_vector_find(const ip4_vector *v, const ip4_address *ip);

void		ip4_vector_free(ip4_vector *v);

/*
  return length, safely
*/
static inline unsigned int ip4_vector_len(register const ip4_vector *v)
{
  if (v->va) return v->len;
  return 0;
}

extern const ip4_address *null_ip4;
extern const ip4_address *localhost_ip4;

#ifdef __cplusplus
}
#endif

#endif
