#include "stdint.h"
#include "fmt.h"
#include "ip4.h"
#include "ip6.h"

unsigned int ip6_fmt(const ip6_address *ip, void *out)
{
  register unsigned int len;
  register unsigned int i;
  register unsigned int j;
  register unsigned int compressing;
  register unsigned int compressed;
  register unsigned long u;
  register char *s;
  register const byte_t *x;
  ip4_address ip4;

  s = out;
  x = ip->d;
  len = compressing = compressed = 0;
  for (j = 0; j < 16; j += 2) {
    if (j == 12 && ip6_is_v4_mapped(ip)) {
      if (compressing) {
        compressing = 0;
      }
      if (out) {
        *s++ = ':';
      }
      ++len;
      ip4_unpack(&ip4, x);
      i = ip4_fmt(&ip4, (out) ? s : FMT_LEN);
      len += i;
      break;
    }
    u = (unsigned long)*x << 8;
    x++;
    u |= (unsigned long)*x;
    x++;
    if (!u && !compressed) {
      if (!compressing) {
        compressing = 1;
        if (j == 0) {
          if (out) {
            *s++ = ':';
          }
          ++len;
        }
      }
    }
    else {
      if (compressing) {
        compressing = 0;
        ++compressed;
        if (out) {
          *s++ = ':';
        }
        ++len;
      }
      i = fmt_xlong(s, u);
      len += i;
      if (out) {
        s += i;
      }
      if (j < 14) {
        if (out) {
          *s++ = ':';
        }
        ++len;
      }
    }
  }
  if (compressing) {
    if (out) {
      *s++ = ':';
    }
    ++len;
  }
  if (ip->sid) {
    if (out) {
      *s++ = '%';
    }
    ++len;
    len += fmt_ulong(s, ip->sid);
  }

  return len;
}
