#include "stdint.h"
#include "scan.h"
#include "ip4.h"
#include "ip6.h"
#include "char.h"

/*
  IPv6 addresses are really ugly to parse!
  Syntax: (h = hex digit, n = base 10 digit)
    1. hhhh:hhhh:hhhh:hhhh:hhhh:hhhh:hhhh:hhhh%nnnnnnnnnn
    2. any number of 0: may be abbreviated as "::", but only once
    3. The last two words may be written as IPv4 address
*/

#define MAX_FIELDS 8

unsigned int ip6_scan(ip6_address *ip, const char *s)
{
  ip4_address ip4;
  byte_t *d;
  const char *x;
  const char *t;
  const char *start;
  const char *field_ptr[MAX_FIELDS];
  unsigned int flag_ip4;
  unsigned int fields;
  unsigned int max_fields;
  unsigned int contraction;
  unsigned int index_contraction;
  unsigned int i;
  unsigned int len;
  unsigned long u;

  ip6_zero(ip);
  start = t = s;
  max_fields = MAX_FIELDS;
  fields = contraction = index_contraction = 0;
/*
  extract field start locations
*/
  while (*t) {
    if (*t == '%') break;
    if (*t == ':') {
      if (fields == MAX_FIELDS) return 0;  /* too many fields */
      field_ptr[fields++] = start;
      t++;  /* skip colon */
      start = t;
      continue;
    }
    if (*t == '.') {  /* IPv4? */
      t++;
      continue;
    }
    if (char_hex_table[(byte_t)*t] >> 4) break;  /* non-hex data */
    t++;
  }
  if (!fields) return 0;  /* syntax error */
/*
  fields = colons + 1
*/
  field_ptr[fields] = start;
  fields++;
/*
  scan for contraction
*/
  for (i = 1; i < fields; i++) {
    x = field_ptr[i];
    if (*x == ':' && *(x - 1) == ':') {  /* adjacent colons */
      contraction++;
      index_contraction = i;
    }
  }
  if (contraction > 1) return 0;  /* only 1 contraction permitted */
/*
  check to see if last field is IPv4 address
*/
  flag_ip4 = 0;
  x = field_ptr[fields - 1];
  len = ip4_scan(&ip4, x);
  if (len) {
    if ((x + len) != t) return 0;  /* format error */
    ip6_copyin_ip4(ip, &ip4);
    max_fields--;  /* 2 become 1 */
    flag_ip4 = 1;
  }
  if (fields > max_fields) return 0;  /* too many fields */
  if (!contraction && fields != max_fields) return 0;  /* insufficient fields */
/*
  now decode upto/including the contraction
*/
  d = ip->d;
  for (i = 0; i < index_contraction; i++) {
    x = field_ptr[i];
    len = scan_xlong(x, &u);
    if (*(x + len) != ':') return 0;  /* format error */
    if (u > 0xffff) return 0;  /* oversize error */
    *d++ = (byte_t)(u >> 8);
    *d++ = (byte_t)(u & 255);
  }
/*
  now decode after the contraction
*/
  i = max_fields - fields;
  d += (i << 1);
  for (i = index_contraction; i < fields - 1; i++) {
    x = field_ptr[i];
    len = scan_xlong(x, &u);
    if (*(x + len) != ':') return 0;  /* format error */
    if (u > 0xffff) return 0;  /* oversize error */
    *d++ = (byte_t)(u >> 8);
    *d++ = (byte_t)(u & 255);
  }
/*
  last field - special handling
*/
  if (!flag_ip4) {
    x = field_ptr[fields - 1];
    len = scan_xlong(x, &u);
    if ((x + len) != t) return 0;  /* format error */
    if (u > 0xffff) return 0;  /* oversize error */
    *d++ = (byte_t)(u >> 8);
    *d++ = (byte_t)(u & 255);
  }
/*
  scope id
*/
  if (*t == '%') {
    ++t;  /* skip '%' */
    len = scan_ulong(t, &u);
    if (!len) return 0;  /* undersize error */
    if (u > 0xffffffff) return 0;  /* oversize error */
    ip->sid = (uint32_t)u;
    t += len;
  }
/*
  return length scanned
*/
  return (unsigned int)(t - s);
}
