#include "stdint.h"
#include "unixtypes.h"
#include "error.h"
#include "stralloc.h"
#include "str.h"
#include "env.h"
#include "unix.h"
#include "alloc.h"
#include "pathexec.h"

static void run(const char *file, char *const *argv, char *const *envp)
{
  static stralloc tmp = STRALLOC;

  const char *path;
  unsigned int split;
  int saved_errno;
/*
  see if prefixed with path
*/
  if (file[str_chr(file, '/')]) {
    execve(file, argv, envp);
    return;
  }
/*
  try PATH entries
*/
  path = env_get("PATH");
  if (!path) {
    path = "/bin:/usr/bin";
  }

  saved_errno = 0;
  for (;;) {
    split = str_chr(path, ':');
    if (!stralloc_copyb(&tmp, path, split)) return;
    if (!split) {
      if (!stralloc_cats(&tmp, ".")) return;
    }
    if (!stralloc_cats(&tmp, "/"))  return;
    if (!stralloc_cats(&tmp, file)) return;
    if (!stralloc_0(&tmp)) return;

    execve(tmp.s, argv, envp);
    if (errno != error_noent) {
      saved_errno = errno;
      if ((errno != error_acces) && (errno != error_perm) && (errno != error_isdir)) return;
    }

    if (!path[split]) {
      if (saved_errno) {
        errno = saved_errno;
      }
      return;
    }
    path += split;
    path++;
  }
}

void pathexec_run(const char *file, char *const *argv, char *const *envp)
{
  char *const *x;
  char **v;
  char *path;
  unsigned int v_len;
  unsigned int pos;
  unsigned int i;

  if (!argv[0]) {  /* safety */
    run(file, argv, envp);
    return;
  }
  pos = str_rchr(argv[0], '/');
  if (!argv[0][pos]) {
    run(file, argv, envp);
    return;
  }
/*
  length of argv
*/
  v_len = 0;
  x = argv;
  while (*x) {
    v_len++;
    x++;
  }
  v_len++;
/*
  make a copy of argv
*/
  v = (char**)alloc(v_len * sizeof(char*));
  if (!v) {  /* failsafe */
    run(file, argv, envp);
    return;
  }
  for (i = 0; i < v_len; i++) {
    v[i] = argv[i];
  }
/*
  fix argv[0] - remove path
*/
  path = v[0];
  path += pos;  /* last '/' */
  path++;
  v[0] = path;
/*
  run with modified argv
*/
  run(file, v, envp);
  alloc_free(v);
}
