#include "stralloc.h"
#include "sa_vector.h"

unsigned int sa_vector_insert(register sa_vector *v, register unsigned int pos, const stralloc *sa)
{
  register unsigned int i;
  stralloc tmp;

  i = sa_vector_len(v);  /* points to new last pos */
  if (!sa_vector_append(v, sa)) return 0;
  if (pos >= i) return 1;  /* shortcut: treat out of bounds as append */
  tmp = v->va[i];  /* save appended data */
  /* move (copy/shuffle) strallocs */
  while (i > pos) {
    v->va[i] = v->va[i - 1];
    i--;
  }
  v->va[pos] = tmp;
  return 1;
}

#ifdef VERSION_3
unsigned int sa_vector_insert(register sa_vector *v, register unsigned int pos, const stralloc *sa)
{
  register unsigned int i;
  stralloc tmp;

  i = sa_vector_len(v);  /* points to new last pos */
  if (pos > i) {  /* fix out of bounds */
    pos = i;
  }
  if (!sa_vector_append(v, sa)) return 0;
  tmp = v->va[i];  /* save preallocated data */
  /* move (copy/shuffle) strallocs */
  while (i > pos) {
    v->va[i] = v->va[i - 1];
    i--;
  }
  v->va[pos] = tmp;
  return 1;
}
#endif

#ifdef VERSION_2
unsigned int sa_vector_insert(register sa_vector *v, register unsigned int pos, const stralloc *sa)
{
  register unsigned int i;

  i = sa_vector_len(v);
  if (pos > i) {  /* fix out of bounds */
    pos = i;
  }
  if (!sa_vector_readyplus(v, 1)) return 0;
  /* move (copy/shuffle) strallocs */
  while (i > pos) {
    v->va[i] = v->va[i - 1];
    i--;
  }
  return stralloc_copy(&(v->va[pos]), sa));
}
#endif

#ifdef VERSION_1
unsigned int sa_vector_insert(register sa_vector *v, register unsigned int pos, const stralloc *sa)
{
  register unsigned int i;
  register unsigned int j;
  stralloc tmp;

  i = sa_vector_len(v);
  if (pos > i) {  /* fix out of bounds */
    pos = i;
  }
  if (!sa_vector_append(v, sa)) return 0;
  while (i > pos) {
    /* sa_vector_swap(v, i, i - 1); */
    j = i - 1;
    tmp = v->va[i];
    v->va[i] = v->va[j];
    v->va[j] = tmp;
    i--;
  }
  return 1;
}
#endif
