#include "stralloc.h"
#include "sa_vector.h"
#include "byte.h"

/*
  Return a list of tokens (token_list) from a string (tokens).
  Delimiters are defined as a set of chars (delimiters)
  Passing of delimiters length allows for parsing of '\0' as a delimiter
*/

unsigned int sa_vector_parse(register sa_vector *token_list, register const stralloc *tokens, register const char *delimiters, register unsigned int dlen)
{
  static stralloc current_token = STRALLOC;
  register unsigned int pos;
  register unsigned int len;
  char ch;

  len = stralloc_len(tokens);
  if (!sa_vector_erase(token_list)) return 0;
  if (!stralloc_erase(&current_token)) return 0;
  for (pos = 0; pos < len; pos++) {
    ch = tokens->s[pos];
    if (byte_chr(delimiters, dlen, ch) == dlen) {  /* not a delimeter */
      if (!stralloc_append(&current_token, &ch)) return 0;
      continue;
    }
    if (!sa_vector_append(token_list, &current_token)) return 0;
    if (!stralloc_erase(&current_token)) return 0;
  }
  if (current_token.len || (byte_chr(delimiters, dlen, ch) < dlen)) {
    if (!sa_vector_append(token_list, &current_token)) return 0;
  }
  return 1;
}

