#include "stdint.h"
#include "char.h"
#include "byte.h"
#include "stralloc.h"
#include "sa_vector.h"
#include "error.h"

/*
  Return a list of tokens (token_list) from a string (tokens).
  It can parse the presence of octal values '\nnn' into single chars
  The delimiter is ':'
   - this can be overruled by enclosing ':' in '[' and ']'
     required for IPv6 addresses
   - or encode ':' as octal, viz: \072
*/

unsigned int sa_vector_parse_config(register sa_vector *token_list, register const stralloc *tokens)
{
  static stralloc current_token = STRALLOC;
  register unsigned int pos;
  register unsigned int len;
  register unsigned int flag_escape;
  register unsigned int flag_octal;
  register unsigned int octal_value;
  register unsigned int value;
  char ch;

  octal_value = flag_octal = flag_escape = 0;
  len = stralloc_len(tokens);
  if (!sa_vector_erase(token_list)) return 0;
  if (!stralloc_erase(&current_token)) return 0;
  for (pos = 0; pos < len; pos++) {
    ch = tokens->s[pos];
    if (flag_octal) {
      value = char_hex_table[(byte_t)ch];
      if (value > 7) {
        errno = error_proto;
        return 0;
      }
      octal_value = (octal_value << 3) | value;
      if (!(--flag_octal)) {
        ch = (char)octal_value;
        if (!stralloc_append(&current_token, &ch)) return 0;
      }
      continue;
    }
    if (ch == '\\') {
      flag_octal = 3;
      octal_value = 0;
      continue;
    }
    if (ch == '[') {
      flag_escape = 1;
    }
    if (flag_escape || ch != ':') {
      if (!stralloc_append(&current_token, &ch)) return 0;
      if (ch == ']') {
        flag_escape = 0;
      }
      continue;
    }
    if (!sa_vector_append(token_list, &current_token)) return 0;
    if (!stralloc_erase(&current_token)) return 0;
  }
  if (flag_escape || flag_octal) {
    errno = error_proto;
    return 0;
  }
  if (current_token.len || ch == ':') {
    if (!sa_vector_append(token_list, &current_token)) return 0;
  }
  return 1;
}

