/*
   SipHash reference C implementation

   Copyright (c) 2012-2014 Jean-Philippe Aumasson
   <jeanphilippe.aumasson@gmail.com>
   Copyright (c) 2012-2014 Daniel J. Bernstein <djb@cr.yp.to>

   To the extent possible under law, the author(s) have dedicated all copyright
   and related and neighboring rights to this software to the public domain
   worldwide. This software is distributed without any warranty.

   You should have received a copy of the CC0 Public Domain Dedication along
   with this software. If not, see
   <http://creativecommons.org/publicdomain/zero/1.0/>.
*/
/*
  Modifications by Stephen A. Welker.
*/

#include "stdint.h"
#include "siphash.h"
#include "uint64.h"

#define ROTL(x, b) (uint64_t)(((x) << (b)) | ((x) >> (64 - (b))))

#define SIPROUND	\
  do {			\
    v0 += v1;		\
    v1 = ROTL(v1, 13);	\
    v1 ^= v0;		\
    v0 = ROTL(v0, 32);	\
    v2 += v3;		\
    v3 = ROTL(v3, 16);	\
    v3 ^= v2;		\
    v0 += v3;		\
    v3 = ROTL(v3, 21);	\
    v3 ^= v0;		\
    v2 += v1;		\
    v1 = ROTL(v1, 17);	\
    v1 ^= v2;		\
    v2 = ROTL(v2, 32);	\
  } while (0)


uint64_t siphash24(const void *in, uint64_t inlen, const byte_t *k)
{
  /* "somepseudorandomlygeneratedbytes" */
  uint64_t v0 = 0x736f6d6570736575ULL;
  uint64_t v1 = 0x646f72616e646f6dULL;
  uint64_t v2 = 0x6c7967656e657261ULL;
  uint64_t v3 = 0x7465646279746573ULL;
  uint64_t b;
  uint64_t k0;
  uint64_t k1;
  uint64_t m;
  unsigned int left = inlen & 7;
  const byte_t *xin = in;
  const byte_t *end = in;

  end += inlen;
  end -= left;

  uint64_unpack(&k0, k);
  uint64_unpack(&k1, k + 8);

  b = ((uint64_t)inlen) << 56;
  v3 ^= k1;
  v2 ^= k0;
  v1 ^= k1;
  v0 ^= k0;

  while (xin != end) {
    uint64_unpack(&m, xin);
    v3 ^= m;
    SIPROUND;
    SIPROUND;
    v0 ^= m;
    xin += 8;
  }

  switch (left) {
    case 7:
      b |= ((uint64_t)xin[6]) << 48;
    case 6:
      b |= ((uint64_t)xin[5]) << 40;
    case 5:
      b |= ((uint64_t)xin[4]) << 32;
    case 4:
      b |= ((uint64_t)xin[3]) << 24;
    case 3:
      b |= ((uint64_t)xin[2]) << 16;
    case 2:
      b |= ((uint64_t)xin[1]) << 8;
    case 1:
      b |= ((uint64_t)xin[0]);
      break;
    case 0:
      break;
    default:
      break;
  }

  v3 ^= b;
  SIPROUND;
  SIPROUND;
  v0 ^= b;
  v2 ^= 0xff;
  SIPROUND;
  SIPROUND;
  SIPROUND;
  SIPROUND;

  return v0 ^ v1 ^ v2 ^ v3;
}
