#ifndef NEMO_UINT32_VECTOR_H
#define NEMO_UINT32_VECTOR_H

#ifdef __cplusplus
extern "C" {
#endif

typedef struct {
  uint32_t *va;
  unsigned int len;
  unsigned int a;
} uint32_vector;

#define UINT32_VECTOR { 0,0,0 }

static inline void uint32_vector_init(register uint32_vector *v)
{
  v->va = 0;
  v->a = v->len = 0;
}

/*  return: 1 = success, 0 = fail  */
unsigned int uint32_vector_realloc(uint32_vector *v, unsigned int n);
unsigned int uint32_vector_alloc(uint32_vector *v, unsigned int n);

static inline unsigned int uint32_vector_ready(register uint32_vector *v, register unsigned int n)
{
  if (v->va) {
    if (n <= v->a) return 1;
    return uint32_vector_realloc(v, n);
  }
  return uint32_vector_alloc(v, n);
}

static inline unsigned int uint32_vector_readyplus(register uint32_vector *v, register unsigned int n)
{
  if (v->va) {
    n += v->len;
    if (n <= v->a) return 1;
    return uint32_vector_realloc(v, n);
  }
  return uint32_vector_alloc(v, n);
}

unsigned int	uint32_vector_copy(uint32_vector *vout, const uint32_vector *vfrom);
unsigned int	uint32_vector_cat(uint32_vector *vout, const uint32_vector *vfrom);

unsigned int	uint32_vector_append(uint32_vector *v, uint32_t d);

unsigned int	uint32_vector_erase(uint32_vector *v);	/* erase contents */
/*
  return index to found position, uiv->len = not found
*/
unsigned int	uint32_vector_find(const uint32_vector *v, uint32_t d);
unsigned int	uint32_vector_rfind(const uint32_vector *v, uint32_t d);
/*
  return length, safely
*/
static inline unsigned int uint32_vector_len(register const uint32_vector *v)
{
  if (v->va) return v->len;
  return 0;
}
/*
  vector manipulation
*/
void	uint32_vector_remove(uint32_vector *v, unsigned int pos, unsigned int len);
void	uint32_vector_swap(uint32_vector *v, unsigned int i, unsigned int j);
void	uint32_vector_sort(uint32_vector *v);

void	uint32_vector_free(uint32_vector *v);
/*
  overwrite with zeroes all allocated space (security - being very paranoid)
*/
void	uint32_vector_zero(uint32_vector *v);
/*
  unicode manipulation
*/
void		uint32_vector_lower(uint32_vector *v);
unsigned int	uint32_vector_utf8_decode(uint32_vector *out, const void *in, unsigned int len);

#ifdef __cplusplus
}
#endif

#endif
