#ifndef NEMO_UNIX_H
#define NEMO_UNIX_H

/*
  common unix routines
*/

#ifdef __cplusplus
extern "C" {
#endif

void	*malloc(size_t size);
void	free(void *ptr);

ssize_t	read(int d, void *buf, size_t nbytes);
ssize_t	write(int d, const void *buf, size_t nbytes);

ssize_t	send(int s, const void *msgbuf, size_t len, int flags);
ssize_t	recv(int s, void *buf, size_t len, int flags);

int	fsync(int fd);
int	close(int fd);

int	ftruncate(int fd, off_t length);
off_t	lseek(int fildes, off_t offset, int whence);

int	dup(int oldd);
int	dup2(int oldd, int newd);

int	pipe(int fildes[2]);

int	rename(const char *from, const char *to);
int	link(const char *name1, const char *name2);
int	unlink(const char *path);
int	symlink(const char *name1, const char *name2);

mode_t	umask(mode_t numask);

int	chown(const char *path, uid_t owner, gid_t group);
int	fchown(int fd, uid_t owner, gid_t group);

int	chmod(const char *path, mode_t mode);
int	fchmod(int fd, mode_t mode);

int	chroot(const char *dirname);

int	chdir(const char *path);
int	fchdir(int fd);
int	mkdir(const char *path, mode_t mode);

int	execl(const char *path, const char *arg, ...);
int	execlp(const char *file, const char *arg, ...);

int	execv(const char *path, char *const argv[]);
int	execvp(const char *file, char *const argv[]);

int	execve(const char *path, char *const argv[], char *const envp[]);

int	gethostname(char *name, size_t namelen);

uid_t	getuid(void);
gid_t	getgid(void);

pid_t	getpid(void);
pid_t	getppid(void);

int	kill(pid_t pid, int sig);

unsigned int	sleep(unsigned int seconds);
unsigned int	alarm(unsigned int seconds);

/*
  non-standard routines that depend on unix environment
*/

time_t		unix_now(void);

uint32_t	unix_cryptorandomnumber(void);

void		unix_randombytes(void *data, size_t len);

#ifdef __cplusplus
}
#endif

#endif
